#---------------------------------------------------------------#
# incith:google                                          v1.5.2 #
#                                                               #
# performs various methods of Google searches                   #
# tested on Eggdrop v1.6.17                                     #
#                                                               #
# Usage:                                                        #
#   .chanset #channel +google                                   #
#   !google [define:|spell:] <search terms> <1+1> <1 cm in ft>  #
#      <patent ##>                                              #
#   !images <search terms>                                      #
#   !local <what> near <where>                                  #
#                                                               #
# ChangeLog:                                                    #
#   1.0: first public release                                   #
#   1.1: improved bolding of search terms, compatible with      #
#          chopped descriptions                                 #
#        supports 'define: <word>' lookups                      #
#        supports calculator. !google (4+3) * 2 - 1             #
#          - converts, too. !google 1 lb in ounces              #
#        image lookups coded, !images <search>                  #
#        'spell: word1 word2' function added                    #
#          - don't rely on this, it's not a dictionary, just    #
#            corrects common typos.                             #
#        flood protection added                                 #
#   1.2: will wrap long lines (yay, a worthy solution!)         #
#        allowed setting of the seperator instead of a ' | ' by #
#          default. If you set this to "\n" then you will get   #
#          each result on a seperate line instead of one line   #
#        will display 'did you mean' if no results              #
#        [PDF] links will be parsed/included now                #
#        fixed a bug when no data was returned from google such #
#          is the case when you search for """"""""""""""""""   #
#   1.3: can return results in multiple languages now           #
#        fixed quotes being displayed around links              #
#        private messages support added (for /msg !google)      #
#        video.google.com seems impossible, Google Video Viewer #
#          is required to view videos, etc                      #
#   1.4: bit of a different output, easier to click links now   #
#        local lookups coded, use !local <what> near <where>    #
#        seems google does currency the same way my exchange    #
#          script does (!g 1 usd in cad)                        #
#        "patent ##" will return the patent link if it exists   #
#        bugfix in private messages                             #
#        sorry to all whom grabbed a borked copy on egghelp :-( #
#   1.5: fix for !local returning html on 'Unverified listing's #
#        "madwoota" has supplied some nice code updates for us! #
#          - "answer" matches, eg: !g population of japan       #
#            - !g <upc code>                                    #
#          - google groups (!gg), google news (!gn)             #
#          - movie: review lookups                              #
#          - area code lookups (!g 780)                         #
#          - print.google lookups (!gp !print)                  #
#        reworked binds to fix horrible bug allowing !g !gi !gp #
#        case insensitive binds (!gi, !GI, !gI, !Gi, etc)       #
#    .1: fix for double triggers/bad binds                      #
#    .2: fix involving "can't read link" error                  #
#                                                               #
# TODO:                                                         #
#   - Suggestions/Thanks/Bugs, e-mail at bottom of header.      #
#                                                               #
# LICENSE:                                                      #
#   This code comes with ABSOLUTELY NO WARRANTY.                #
#                                                               #
#   This program is free software; you can redistribute it      #
#   and/or modify it under the terms of the GNU General Public  #
#   License as published by the Free Software Foundation;       #
#   either version 2 of the License, or (at your option) any    #
#   later version.                                              #
#                                                               #
#   This program is distributed in the hope that it will be     #
#   useful, but WITHOUT ANY WARRANTY; without even the implied  #
#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR     #
#   PURPOSE.  See the GNU General Public License for more       #
#   details. (http://www.gnu.org/copyleft/library.txt)          #
#                                                               #
# Copyright (C) 2005, Jordan                                    #
# incith@gmail.com                                              #
#---------------------------------------------------------------#
package require http 2.3
setudef flag google

# 0 will typically disable an option, otherwise a value 1 or
# above will enable it.
#
namespace eval incith {
  namespace eval google {
    # set this to the command character you want to use for the binds
    variable command_char "!"

    # set these to your preferred binds ("one two")
    variable google_binds "g google"
    variable image_binds "gi image images"
    variable local_binds "gl local"
    variable group_binds "gg group groups"
    variable news_binds "gn news"
    variable print_binds "gp print"

    # if you want to allow users to search via /msg, enable this
    variable private_messages 1

    # set this to the language you want results in! use 2 letter form.
    #   "all" is the default/standard google.com search.
    #   See http://www.google.com/advanced_search for a list.  You have to use
    #   the 'Language' dropdown box, perform a search, and find a line in the URL
    #   that looks like "&lr=lang_en" (for English). "en" is your language code.
    # Popular Ones: it (italian), da (danish), de (german), es (spanish), fr (french)
    # please note, this does not 'translate', it searches Google in a
    #   language of choice, which means you can still get English results.
    variable language "all"

    # number of search results/image links to return, 'define:' is always 1 as some defs are huge
    variable search_results 3
    variable image_results 4
    variable local_results 4
    variable group_results 3
    variable news_results 3
    variable print_results 3

    # what to use to seperate results, set this to "\n" and it will output each result
    #   on a line of its own. the seperator will be removed from the end of the last result.
    variable seperator " | "

    # ** this is not an optional setting, if a string is too long to send, it won't be sent! **
    # It should be set to the max amount of characters that will be received in a public
    #   message by your IRC server.  If you find you aren't receiving results, try lowering this.
    variable split_length 443

    # trimmed length of returned descriptions, only for standard searches.
    variable description_length 40

    # replace search terms appearing in the description as bolded words?
    variable bold_descriptions 1

    # underline links returned from search results?
    variable underline_links 0

    # number of minute(s) to ignore flooders, 0 to disable flood protection
    variable ignore 1

    # how many requests in how many seconds is considered flooding?
    # by default, this allows 3 queries in 10 seconds, the 4th being ignored
    #   and ignoring the flooder for 'variable ignore' minutes
    variable flood 4:10
  }
}

# end of configuration, script begins
namespace eval incith {
  namespace eval google {
    variable version "incith:google-1.5.2"
  }
}

# bind the public binds
bind pubm -|- "*" incith::google::public_message

# bind the private message binds, if wanted
if {$incith::google::private_messages >= 1} {
  bind msgm -|- "*" incith::google::private_message
}

namespace eval incith {
  namespace eval google {
    # GOOGLE
    # performs a search on google.
    #
    proc google {input} {
      # local variable initialization
      ; set results 0 ; set calc 0 ; set spell 0 ; set output ""

      # can't be moved to fetch_html since $spell isn't global
      if {[string match -nocase "spell:*" $input] == 1} {
        set spell 1
      }

      # if we don't want any search results, stop now.
      if {$incith::google::search_results <= 0} {
        return
      }

      # fetch the html
      set html [fetch_html $input 1]

      # strip out 'did you mean?' first
      regexp -nocase -- {Did you mean: <a href.+?>(.+?)</a>} $html - did_you_mean
      if {[info exists did_you_mean] == 1} {
        regsub -all -- "\002" $did_you_mean {} did_you_mean
      }

      # parse the html
      while {$results < $incith::google::search_results} {
        # the regexp grabs the data and stores them into their variables, while
        # the regsub removes the data, for the next loop to grab something new.
        #
        # top match / "answer" match
        if {[string match "*&oi=answers*" $html] == 1} {
          regexp -- {oi=answers(?:.+?)=(.+?)">(.+?)<br>} $html - link desc
          regsub -- {oi=answers(.+?)<br>} $html {} html
          regsub -all "(</a>)" $desc "" desc
        # area codes
        } elseif {[string match "*/images/map_icon2.gif*" $html] == 1} {
          regexp -- {Show map of area code (.+?)</a>} $html - area_code
          regexp -- {/images/map_icon2.gif(?:.+?)<a href="(.+?)"(?:.+?)<br>(.+?)</a>} $html - link desc
          if {[info exists area_code] == 1} {
            set desc "${area_code}: ${desc}"
            regsub -- {Show map of area code (.+?)</a>} $html {} html
          }
          regsub -- {/images/map_icon2.gif(.+?)</a>} $html {} html
        # define:
        } elseif {[string match "define:*" $input] == 1} {
          regexp -- {<li>(.+?)(?=\s*<li>|<br>).*<a href.*(http.+?)">} $html - desc link
          regsub -- {<li>(.+?)(?=\s*<li>|<br>)} $html {} html
          regsub -all -- {\+} $input {%2B} define_input
          regsub -all -- { } $define_input {+} define_input
          append link " ( http://www.google.com/search?hl=${incith::google::language}&q=${define_input} )"
        # movie:
        } elseif {[string match "movie:*" $input] == 1} {
          regexp -- {<td valign=top><a href="/movies/reviews\?cid=(.+?)&fq(?:.+?)">(.+?)</a>} $html - cid desc
          regsub -- {<td valign=top><a href="/movies/reviews\?cid=(.+?)</a>} $html {} html
          set link "http://www.google.com/movies/reviews?cid=${cid}"
        # patents
        } elseif {[regexp {^patent\s+(\d+)\s*$} $input]} {
          #regexp -- {<a href="(http://patft.uspto.gov/.+?)">(.+?)</a>} $html - link desc
          #regsub -- {<a href="(http://patft.uspto.gov/.+?)">(.+?)</a>} $html {} html
          set calc 1
          regexp -- {<a href="(http://patft.uspto.gov/.+?)">} $html - desc
          regsub -- {<a href="(http://patft.uspto.gov/.+?)">} $html {} html
          set link "Iteration: $results"
        # spell: checker
        } elseif {$spell == 1} {
          if {[info exists did_you_mean] == 1} {
    	  # no, I didn't change this 'seperator' into the global setting
            regsub -all -- { } $did_you_mean {, } desc
            regsub -- {^(.*)} $desc {Did You Mean: \1} desc
          } else {
            set desc "No alternate spellings were found."
          }
          set link "Iteration: $results"
        # calculator
        } elseif {[string match "*<img src=/images/calc_img*" $html] == 1} {
          set calc 1
          # remove bold codes from the html, not necessary here.
          regsub -all -- "\002" $html {} html
          regexp -nocase -- {calc_img.+?nowrap>(.+?)</} $html - desc
          set link "Iteration: $results"
        # regular search
        } else {
          regexp -- {<p class=g><a href=(.+?)>(.+?)</a>} $html - link desc
          if {[info exists link] == 1} {
            if {[string match "*/url\?sa=U*" $link] == 1} {
              regexp -- {/url\?sa=U(?:.+?)q=(.*)&e=} $link - link
            } elseif {[string match *.pdf $link] == 1} {
	      set desc "\[PDF\] $desc"
            }
	  }
          regsub -- {<p class=g><a href=(.+?)>(.+?)</a>} $html {} html
        }

        # if there's no link or description, return or stop looping, depending
        if {[info exists link] == 0 || [info exists desc] == 0} {
          if {$results == 0} {
	    set reply "Sorry, no search results were found."
	    if {[info exists did_you_mean] == 1} {
              append reply "  Did you mean: ${did_you_mean}?"
	    }
	    return $reply
	  } else {
	    break
	  }
	}

        # prevent duplicate results, somewhat. If result 1 is 'foo.com/bar/' and
        #   result 2 is 'foo.com', result 2 will be skipped.
        if {[string match "*$link*" $output] == 1 && $calc == 0 && $spell == 0} {
          continue
        }

        # trim the description
        if {$incith::google::description_length > 0 && [string match "define:*" $input] == 0 && $calc == 0 && $spell == 0} {
          set desc [string range $desc 0 [expr $incith::google::description_length - 1]]
        }

        # this removes trailing spaces from the description, and converts %20's to spaces
        # \017 is ^O, read by clients as 'stop bold/colors/underline'
        set desc [string trimright $desc]
        regsub -all "%20" $desc " " desc
        append desc "\017"

        # underline links
        if {$incith::google::underline_links > 0 && $calc == 0 && $spell == 0} {
          set link "\037${link}\037"
        }

        # add the search result
        if {$calc == 0 && $spell == 0} {
	  regsub -all "(?:\x93|\x94|&quot;|\")" $link {} link
	  append output "${desc}\017 \@ ${link}${incith::google::seperator}"
        } else {
          append output "${desc}\017"
          break
        }

        # I've hard-coded it to only fetch one result for define lookups, and
        #   spell shouldn't ever have to loop.
        if {[string match "define:*" $input] == 1 || $spell == 1} {
          break
        }

        # increase the results, clear the variables for the next loop
        unset link ; unset desc ; set spell 0 ; set calc 0
        incr results
      }

      # make sure we have something to send
      if {[info exists output] == 0} {
        set reply "Sorry, no search results were found."
        if {[info exists did_you_mean] == 1} {
          append reply "  Did you mean: ${did_you_mean}?"
	}
        return $reply
      }
      return $output
    }

    # IMAGES
    # fetches a list of links to images from images.google.
    #
    proc images {input} {
      ; set results 0 ; set output ""

      # if we don't want any search results, stop now.
      if {$incith::google::image_results <= 0} {
        return
      }

      # fetch the html
      set html [fetch_html $input 2]

      # strip out 'did you mean?' first
      regexp -nocase -- {Did you mean: <a href.+?>(.+?)</a>} $html - did_you_mean
      if {[info exists did_you_mean] == 1} {
        regsub -all -- "\002" $did_you_mean {} did_you_mean
      }

      # parse the html
      while {$results < $incith::google::image_results} {
        regexp -- {<a href=/imgres\?imgurl=(.+?)&imgrefurl} $html - link
        regsub -- {<a href=/imgres\?imgurl=(.+?)&imgrefurl} $html {} html
  
        # if there's no link, return or stop looping, depending
        if {![info exists link]} {
          if {$results == 0} {
            set reply "Sorry, no search results were found."
            if {[info exists did_you_mean] == 1} {
              append reply "  Did you mean: ${did_you_mean}?"
	    }
	    return $reply
          } else {
            break
          }
        }

        # prevent duplicate results is mostly useless here, but will at least
        #   ensure that we don't get the exact same picture.
        if {[string match "*$link*" $output] == 1} { 
	  continue
        }

        # underline links?
        if {$incith::google::underline_links > 0} {
          set link "\037${link}\037"
        }

        # add the search result
        # \017 is ^O, read by clients as 'reset color' (stop bold/etc)
        append output "${link}\017${incith::google::seperator}"

        # increase the results, clear the variables for the next loop just in case
        unset link
        incr results
      }

      # make sure we have something to send
      if {[info exists output] == 0} {
        set reply "Sorry, no search results were found."
        if {[info exists did_you_mean] == 1} {
          append reply "  Did you mean: ${did_you_mean}?"
	}
        return $reply
      }
      return $output
    }

    # LOCAL
    # fetches a list of address & phone numbers from local.google.
    #
    proc local {input} {
      ; set results 0 ; set output ""

      # if we don't want any search results, stop now.
      if {$incith::google::local_results <= 0} {
        return
      }

      # fetch the html
      set html [fetch_html $input 3]

      # strip out 'did you mean?' first
      regexp -nocase -- {Did you mean: <a href.+?>(.+?)</a>} $html - did_you_mean
      if {[info exists did_you_mean] == 1} {
        regsub -all -- "\002" $did_you_mean {} did_you_mean
      }

      # parse the html
      while {$results < $incith::google::local_results} {
        regexp -- {<a href="/local\?q=.*?">(.+?)</a><br><nobr>(.+?)</nobr></td><td valign=top class=lad>(.+?)<br>} $html - loc num add1
        regsub -- {<a href="/local\?q=.*?">(.+?)</a><br><nobr>(.+?)</nobr></td><td valign=top class=lad>(.+?)<br>} $html {} html
  
        # if there's no link, return or stop looping, depending
        if {[info exists loc] == 0} {
          if {$results == 0} {
            set reply "Sorry, no search results were found."
            if {[info exists did_you_mean] == 1} {
              append reply "  Did you mean: ${did_you_mean}?"
	    }
	    return $reply
          } else {
            break
          }
        }

        # add the search result
        # \017 is ^O, read by clients as 'reset color' (stop bold/etc)
        append output "${loc}\017 \@ ${add1}, Ph. ${num}\017${incith::google::seperator}"

        # increase the results, clear the variables for the next loop just in case
        unset loc
        incr results
      }

      # make sure we have something to send
      if {[info exists output] == 0} {
        set reply "Sorry, no search results were found."
        if {[info exists did_you_mean] == 1} {
          append reply "  Did you mean: ${did_you_mean}?"
	}
        return $reply
      }

      return $output
    }

    # GROUPS
    # fetches a list of threads from groups.google.
    #
    proc groups {input} {
      ; set results 0 ; set output ""

      # if we don't want any search results, stop now.
      if {$incith::google::group_results <= 0} {
        return
      }

      # fetch the html
      set html [fetch_html $input 4]

      # strip out 'did you mean?' first
      regexp -nocase -- {Did you mean: <a href.+?>(.+?)</a>} $html - did_you_mean
      if {[info exists did_you_mean] == 1} {
        regsub -all -- "\002" $did_you_mean {} did_you_mean
      }

      # parse the html
      while {$results < $incith::google::group_results} {
        regexp -- {<td class="r">(?:.+?)/group/(.+?)/browse_thread/thread/(.+?)/(?:.+?)">(.+?)</a>} $html {} group thread desc
        regsub -- {<td class="r">(.+?)</a>} $html {} html

        # if there's no desc, return or stop looping, depending
        if {[info exists desc] == 0} {
          if {$results == 0} {
            set reply "Sorry, no search results were found."
            if {[info exists did_you_mean] == 1} {
              append reply "  Did you mean: ${did_you_mean}?"
            }
            return $reply
          } else {
            break
          }
        }

        # make the link valid because we only got a partial href result, not a full url
        set link "http://groups.google.com/group/${group}/browse_thread/thread/${thread}"

        # prevent duplicate results is mostly useless here, but will at least
        #   ensure that we don't get the exact same picture.
        if {[string match "*$link*" $output] == 1} { 
          continue
        }

        # underline links?
        if {$incith::google::underline_links > 0} {
          set link "\037${link}\037"
        }

        # add the search result
        append output "${desc}\017 \@ ${link}${incith::google::seperator}"        

        # increase the results, clear the variables for the next loop just in case
        unset link; unset desc
        incr results
      }

      # make sure we have something to send
      if {[info exists output] == 0} {
        set reply "Sorry, no search results were found."
        if {[info exists did_you_mean] == 1} {
          append reply "  Did you mean: ${did_you_mean}?"
        }
        return $reply
      }
      return $output
    }

    # NEWS
    # fetches the news from news.google.
    # -madwoota supplied
    #
    proc news {input} {
      ; set results 0 ; set output ""

      # if we don't want any search results, stop now.
      if {$incith::google::news_results <= 0} {
        return
      }

      # fetch the html
      set html [fetch_html $input 5]
      
      # strip out 'did you mean?' first
      regexp -nocase -- {Did you mean: <a href.+?>(.+?)</a>} $html - did_you_mean
      if {[info exists did_you_mean] == 1} {
        regsub -all -- "\002" $did_you_mean {} did_you_mean
      }

      # We have to do a single regsub first to get down to the rough area in the html we need because
      # its just *not* very elegant attempting doing this in a single regexp. (Read: unreliable)
      regsub -- {<html>(.+?)<div class=mainbody>} $html {} html

      # parse the html
      while {$results < $incith::google::news_results} {
        # somewhat extenuated regexp due to allowing that there *might* be
        # an image next to the story
        regexp -- {<table border=0(?:.+?)<td valign=top><a href="(.+?)"(?:.+?)>(.+?)</a>} $html {} link desc 
        regsub -- {<table border=0(.+?)</a>} $html {} html

        # if there's no desc, return or stop looping, depending
        if {[info exists desc] == 0} {
          if {$results == 0} {
            set reply "Sorry, no search results were found."
            if {[info exists did_you_mean] == 1} {
              append reply "  Did you mean: ${did_you_mean}?"
            }
            return $reply
          } else {
            break
          }
        }

        # prevent duplicate results is mostly useless here, but will at least
        #   ensure that we don't get the exact same article.
        if {[string match "*$link*" $output] == 1} { 
          continue
        }

        # underline links?
        if {$incith::google::underline_links > 0} {
          set link "\037${link}\037"
        }

        # add the search result
        append output "${desc}\017 \@ ${link}${incith::google::seperator}"        

        # increase the results, clear the variables for the next loop just in case
        unset link; unset desc
        incr results
      }

      # make sure we have something to send
      if {[info exists output] == 0} {
        set reply "Sorry, no search results were found."
        if {[info exists did_you_mean] == 1} {
          append reply "  Did you mean: ${did_you_mean}?"
        }
        return $reply
      }
      return $output
    }

    # PRINT
    # fetches book titles from print.google.
    # -madwoota supplied
    #
    proc print {input} {
      ; set results 0 ; set output ""

      # if we don't want any search results, stop now.
      if {$incith::google::print_results <= 0} {
        return
      }

      # fetch the html
      set html [fetch_html $input 6]

      # strip out 'did you mean?' first
      regexp -nocase -- {Did you mean: <a href.+?>(.+?)</a>} $html - did_you_mean
      if {[info exists did_you_mean] == 1} {
        regsub -all -- "\002" $did_you_mean {} did_you_mean
      }

      # We have to do a single regsub first to get down to the rough area in the html we need because
      # its just *not* very elegant attempting doing this in a single regexp. (Read: unreliable)
      regsub -- {<html>(.+?)style="padding} $html {} html

      # parse the html
      while {$results < $incith::google::print_results} {
        # somewhat extenuated regexp due to allowing that there might be an image next to the title
        regexp -- {<p class="e">(?:.+?)href="(.+?)&(?:.+?)">(.+?)</a>} $html {} link desc
        regsub -- {<p class="e">(.+?)</a>} $html {} html

        # if there's no desc, return or stop looping, depending
        if {[info exists desc] == 0} {
          if {$results == 0} {
            set reply "Sorry, no search results were found."
            if {[info exists did_you_mean] == 1} {
              append reply "  Did you mean: ${did_you_mean}?"
            }
            return $reply
          } else {
            break
          }
        }

        # prevent duplicate results is mostly useless here, but will at least
        #   ensure that we don't get the exact same article.
        if {[string match "*$link*" $output] == 1} {
         continue
        }

        # underline links?
        if {$incith::google::underline_links > 0} {
          set link "\037${link}\037"
        }

        # add the search result
        append output "${desc}\017 \@ ${link}${incith::google::seperator}"

        # increase the results, clear the variables for the next loop just in case
        unset link; unset desc
        incr results
      }

      # make sure we have something to send
      if {[info exists output] == 0} {
        set reply "Sorry, no search results were found."
        if {[info exists did_you_mean] == 1} {
          append reply "  Did you mean: ${did_you_mean}?"
       }
        return $reply
      }
      return $output
    }

    # FETCH_HTML
    # fetches html for the various *.google.com sites
    #
    proc fetch_html {input switch} {
      # for local
      regexp -nocase -- {^(.+?) near (.+?)$} $input - search location
      # for the rest
      regsub -all -- {\+} $input {%2B} input
      regsub -all -- { } $input {+} input

      # GOOGLE
      if {$switch == 1} {
        # we don't want 'define:+<search>', so we'll just remove the space if there is one.
        regsub -nocase -- {^define:\s*} $input {define:} input

        # spell after define so 'spell: define: foo' doesn't turn into a define lookup
        if {[string match -nocase "spell:*" $input] == 1} {
          regsub -nocase -- {^spell:\s*} $input {} input
        }

        if {[string match "movie:*" $input] == 1} {
          set query "http://www.google.com/movies?hl=en&q=${input}&safe=off&btnG=Search&lr=lang_${incith::google::language}"
        } else {
          set query "http://www.google.com/search?hl=en&q=${input}&safe=off&btnG=Search&lr=lang_${incith::google::language}"
        }
      # IMAGES
      } elseif {$switch == 2} {
        set query "http://images.google.com/images?hl=en&safe=off&q=${input}&safe=off&btnG=Search+Images"
      # LOCAL
      } elseif {$switch == 3} {
        # a + joins words together in the search, so we change +'s to there search-form value
        regsub -all -- {\+} $search {%2B} search
        regsub -all -- {\+} $location {%2B} location
        # change spaces to +'s for a properly formatted search string.
        regsub -all -- { } $search {+} search
        regsub -all -- { } $location {+} location

        set query "http://local.google.com/local?hl=en&q=${search}&near=${location}&btnG=Google+Search"
      } elseif {$switch == 4} {
        set query "http://groups.google.com/groups?hl=en&q=${input}&safe=off"
      } elseif {$switch == 5} {
        set query "http://news.google.com/news?hl=en&q=${input}&lr=&sa=N&tab=gn"
      } elseif {$switch == 6} {
        set query "http://print.google.com/print?q=${input}&btnG=Search+Print"
      }

      # beware, changing the useragent will result in differently formatted html from Google.
      set ua "Lynx/2.8.5rel.1 libwww-FM/2.14 SSL-MM/1.4.1 OpenSSL/0.9.7e"
      set http [::http::config -useragent $ua]
      set http [::http::geturl $query -timeout [expr 1000 * 10]]
      set html [::http::data $http]

      # generic pre-parsing
      regsub -all "\n" $html "" html
      regsub -all "(?:\x91|\x92|&#39;)" $html {'} html
      regsub -all "(?:\x93|\x94|&quot;)" $html {"} html
      regsub -all "&amp;" $html {\&} html
      regsub -all -nocase {<sup>(.+?)</sup>} $html {^\1} html
      regsub -all -nocase {<font.+?>} $html "" html
      regsub -all -nocase {</font>} $html "" html
      regsub -all -nocase {<span.+?>} $html "" html
      regsub -all -nocase {</span>} $html "" html
      regsub -all -nocase {<input.+?>} $html "" html
      regsub -all -nocase {(?:<i>|</i>)} $html "" html
      # this is the "---" line in "population of Japan" searches
      regsub -all "&#8212;" $html "--" html
      regsub -all "\n" $html "" html
      regsub -all "&times;" $html {*} html
      regsub -all "&nbsp;" $html { } html
      regsub -all -nocase "&#215;" $html "x" html
      regsub -all -nocase "&lt;" $html "<" html
      regsub -all -nocase "&gt;" $html ">" html

      # regexps that should remain seperate go here
      # google specific regexps
      if {$switch == 1} {
        # regexp the rest of the html for a much easier result to parse
        regsub -all -nocase {<b>\[PDF\]</b>\s*} $html "" html
      } elseif {$switch == 2} {
        # these %2520 codes, I have no idea. But they're supposed to be %20's
        regsub -all {%2520} $html {%20} html
      } elseif {$switch == 3} {
        regsub -all -nocase { - <nobr>Unverified listing</nobr>} $html "" html
      } elseif {$switch == 4} {
      } elseif {$switch == 5} {
      } elseif {$switch == 6} {
      }
      # no point having it so many times
      if {$incith::google::bold_descriptions > 0} {
        regsub -all -nocase {(?:<b>|</b>)} $html "\002" html
      }

      return $html
    }

    # PUBLIC_MESSAGE
    # decides what to do with binds that get triggered
    #
    proc public_message {nick uhand hand chan input} {
      if {[lsearch -exact [channel info $chan] +google] != -1} {
        send_output $input $chan $nick $uhand
      }
    }

    # PRIVATE_MESSAGE
    # decides what to do with binds that get triggered
    #
    proc private_message {nick uhand hand input} {
      if {$incith::google::private_messages >= 1} {
	send_output $input $nick $nick $uhand
      }
    }

    # SEND_OUTPUT
    # no point having two copies of this in public/private_message{}
    #
    proc send_output {input where nick uhand} {
      # check for !google
      foreach bind [split $incith::google::google_binds " "] {
        if {[regexp -nocase -- "^${incith::google::command_char}${bind}\\s+.+" $input] == 1} {
          # flood protection check
          if {[flood $nick $uhand]} {
            return
          }
          # call google
          regexp -nocase -- "^${incith::google::command_char}${bind}\\s+(.*)" $input - search
          foreach line [incith::google::parse_output [google $search]] {
            putquick "PRIVMSG $where :$line"
	  }
	  break
	}
      }
      # check for !images
      foreach bind [split $incith::google::image_binds " "] {
        if {[regexp -nocase -- "^${incith::google::command_char}${bind}\\s+.+" $input] == 1} {
          # flood protection check
          if {[flood $nick $uhand]} {
            return
          }
          # call images
          regexp -nocase -- "^${incith::google::command_char}${bind}\\s+(.*)" $input - search
          foreach line [incith::google::parse_output [images $search]] {
            putquick "PRIVMSG $where :$line"
          }
          break
        }
      }
      # check for !local
      foreach bind [split $incith::google::local_binds " "] {
        if {[regexp -nocase -- "^${incith::google::command_char}${bind}\\s+.+" $input] == 1} {
          # flood protection check
          if {[flood $nick $uhand]} {
            return
          }
          # call local
          regexp -nocase -- "^${incith::google::command_char}${bind}\\s+(.*)" $input - search

          # local requires splitting of the search
          regexp -nocase -- {^(.+?) near (.+?)$} $search - what location

          if {![info exists what] || ![info exists location]} {
            putquick "PRIVMSG $where :Local searches should be the format of 'pizza near footown, bar'"
            return
          }

          foreach line [incith::google::parse_output [local $search]] {
            putquick "PRIVMSG $where :$line"
          }
          break
        }
      }
      # check for !groups
      foreach bind [split $incith::google::group_binds " "] {
        if {[regexp -nocase -- "^${incith::google::command_char}${bind}\\s+.+" $input] == 1} {
          # flood protection check
          if {[flood $nick $uhand]} {
            return
          }
          # call groups
          regexp -nocase -- "^${incith::google::command_char}${bind}\\s+(.*)" $input - search
          foreach line [incith::google::parse_output [groups $search]] {
            putquick "PRIVMSG $where :$line"
          }
          break
        }
      }
      # check for !news
      foreach bind [split $incith::google::news_binds " "] {
        if {[regexp -nocase -- "^${incith::google::command_char}${bind}\\s+.+" $input] == 1} {
          # flood protection check
          if {[flood $nick $uhand]} {
            return
          }
          # call news
          regexp -nocase -- "^${incith::google::command_char}${bind}\\s+(.*)" $input - search
          foreach line [incith::google::parse_output [news $search]] {
            putquick "PRIVMSG $where :$line"
          }
          break
        }
      }
      # check for !print
      foreach bind [split $incith::google::print_binds " "] {
        if {[regexp -nocase -- "^${incith::google::command_char}${bind}\\s+.+" $input] == 1} {
          # flood protection check
          if {[flood $nick $uhand]} {
            return
          }
          # call print
          regexp -nocase -- "^${incith::google::command_char}${bind}\\s+(.*)" $input - search
          foreach line [incith::google::parse_output [print $search]] {
            putquick "PRIVMSG $where :$line"
          }
          break
        }
      }
    }

    # PARSE_OUTPUT
    # prepares output for sending to a channel/user, calls line_wrap
    #
    proc parse_output {input} {
      set parsed_output [set parsed_current {}]
      if {[string match "\n" $incith::google::seperator] == 1} {
        regsub {\n\s*$} $input "" input
        foreach newline [split $input "\n"] {
          foreach line [incith::google::line_wrap $newline] {
            lappend parsed_output $line
          }
        }
      } else {
        regsub "(?:${incith::google::seperator}|\\|)\\s*$" $input {} input
        foreach line [incith::google::line_wrap $input] {
          lappend parsed_output $line
        }
      }
      return $parsed_output
    }  

    # LINE_WRAP
    # takes a long line in, and chops it before the specified length
    # http://forum.egghelp.org/viewtopic.php?t=6690
    #
    proc line_wrap {str {splitChr { }}} { 
      set out [set cur {}]
      set i 0
      set len $incith::google::split_length
      #regsub -all "\002" $str "<ZQ" str
      #regsub -all "\037" $str "<ZX" str
      foreach word [split [set str][set str ""] $splitChr] { 
        if {[incr i [string len $word]] > $len} { 
          #regsub -all "<ZQ" $cur "\002" cur
          #regsub -all "<ZX" $cur "\037" cur
          lappend out [join $cur $splitChr] 
          set cur [list $word] 
          set i [string len $word] 
        } else { 
          lappend cur $word 
        } 
        incr i 
      } 
      #regsub -all "<ZQ" $cur "\002" cur
      #regsub -all "<ZX" $cur "\037" cur
      lappend out [join $cur $splitChr] 
    }

    # FLOOD_INIT
    # modified from bseen
    #
    variable flood_data
    variable flood_array
    proc flood_init {} {
      if {$incith::google::ignore < 1} {
        return 0
      }
      if {![string match *:* $incith::google::flood]} {
        putlog "$incith::google::version: variable flood not set correctly."
        return 1
      }
      set incith::google::flood_data(flood_num) [lindex [split $incith::google::flood :] 0]
      set incith::google::flood_data(flood_time) [lindex [split $incith::google::flood :] 1]
      set i [expr $incith::google::flood_data(flood_num) - 1]
      while {$i >= 0} {
        set incith::google::flood_array($i) 0
        incr i -1
      }
    }
    ; flood_init

    # FLOOD
    # updates and returns a users flood status
    #
    proc flood {nick uhand} {
      if {$incith::google::ignore < 1} {
        return 0
      }
      if {$incith::google::flood_data(flood_num) == 0} {
        return 0
      }
      set i [expr ${incith::google::flood_data(flood_num)} - 1]
      while {$i >= 1} {
        set incith::google::flood_array($i) $incith::google::flood_array([expr $i - 1])
        incr i -1
      }
      set incith::google::flood_array(0) [unixtime]
      if {[expr [unixtime] - $incith::google::flood_array([expr ${incith::google::flood_data(flood_num)} - 1])] <= ${incith::google::flood_data(flood_time)}} {
        putlog "$incith::google::version: flood detected from ${nick}."
        newignore [join [maskhost *!*[string trimleft $uhand ~]]] $incith::google::version flooding $incith::google::ignore
        return 1
      } else {
        return 0
      }
    }
  }
}

putlog " - $incith::google::version loaded."

# EOF
